@@ -5,16 +5,26 @@
 # license:   perl
 # copyright: 2011
 
-package Pegex::Compiler::Receiver;
-use Pegex::Receiver -base;
+package Pegex::Compiler::AST;
+use Pegex::Mo;
+extends 'Pegex::Receiver';
 
 use Pegex::Grammar::Atoms;
 
 has 'top';
-has 'extra_rules' => {};
+has 'extra_rules' => default => sub {+{}};
+use constant wrap => 1;
+
+my %prefixes = (
+    '!' => ['+asr', -1],
+    '=' => ['+asr', 1],
+    '.' => '-skip',
+    '-' => '-pass',
+    '+' => '-wrap',
+);
 
 # Uncomment this to debug. See entire raw AST.
-# sub final {
+# sub finalize {
 #     my ($self, $match) = @_;
 #     XXX $match;
 # }
@@ -23,14 +33,12 @@ has 'extra_rules' => {};
 
 
 sub got_grammar {
-    my ($self, $match) = @_;
+    my ($self, $rules) = @_;
     my $grammar = {
         '+top' => $self->top,
         %{$self->extra_rules},
     };
-    my $rules = $match->[0];
     for (@$rules) {
-        $_ = $_->[1];
         my ($key, $value) = %$_;
         $grammar->{$key} = $value;
     }
@@ -39,17 +47,21 @@ sub got_grammar {
 
 sub got_rule_definition {
     my ($self, $match) = @_;
-    my $name = $match->[1]{rule_name}{1};
+    my $name = $match->[0]{rule_name};
+    $self->{top} = $name if $name eq 'TOP';
     $self->{top} ||= $name;
-    my $value = $match->[3]{rule_group};
+    my $value = $match->[1]{rule_group};
     return +{ $name => $value };
 }
 
 sub got_bracketed_group {
     my ($self, $match) = @_;
     my $group = $match->[1]{rule_group};
-    if (my $mod = $match->[2]{1}) {
-        $group->{'+mod'} = $mod;
+    if (my $prefix = $match->[0]) {
+        $group->{$prefixes{$prefix}} = 1;
+    }
+    if (my $qty = $match->[-1]) {
+        $group->{'+qty'} = $qty;
     }
     return $group;
 }
@@ -88,28 +100,39 @@ sub get_group {
     return [ get($group) ];
 }
 
+sub got_rule_part {
+    my ($self, $part) = @_;
+    my ($rule, $sep) = @$part;
+    if ($sep) {
+        $rule->{rule_item}{'.sep'} = $sep->[0]{rule_item};
+    }
+    return $rule;
+}
+
 sub got_rule_reference {
     my ($self, $match) = @_;
-    my ($assertion, $ref, $quantifier) =
-        @{$match}{qw(1 2 3)};
+    my ($prefix, $ref, $suffix) = @$match;
     my $node = +{ '.ref' => $ref };
     if (my $regex = Pegex::Grammar::Atoms->atoms->{$ref}) {
         $self->extra_rules->{$ref} = +{ '.rgx' => $regex };
     }
-    if (my $mod = $assertion || $quantifier) {
-        $node->{'+mod'} = $mod;
+    $node->{'+qty'} = $suffix if $suffix;
+    if ($prefix) {
+        my ($key, $val) = ($prefixes{$prefix}, 1);
+        ($key, $val) = @$key if ref $key;
+        $node->{$key} = $val;
     }
     return $node;
 }
 
 sub got_regular_expression {
     my ($self, $match) = @_;
-    return +{ '.rgx' => $match->{1} };
+    return +{ '.rgx' => $match };
 }
 
 sub got_error_message {
     my ($self, $match) = @_;
-    return +{ '.err' => $match->{1} };
+    return +{ '.err' => $match };
 }
 
 1;
